# -*- coding: utf-8 -*-

import time
import datetime
import re

import dateutil
import dateutil.parser
from sp_api.base.exceptions import SellingApiForbiddenException, SellingApiBadRequestException

from dropshipping.spapi.exceptions import SellingApiInvalidAsinException

from em_tasks import logger
from em_tasks.spapi.spapi_catalog_items_parser import SpapiCatalogItemsParser
from em_tasks.spapi import exceptions_to_retry, exceptions_not_retry


class SpapiUpdateCatalogItemsTask():
  def __init__(self, spapi, product_service, marketplace, asins, error_service=None):
    self.spapi = spapi
    self.product_service = product_service
    self.error_service = error_service
    self.marketplace = marketplace.lower()
    self.asins = asins

  def run(self):
    cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
    indice_name = 'amz_products_api_{}'.format(self.marketplace)

    products_info = None
    while True:
      try:
        products_info = self.search_catalog_items(self.spapi, self.marketplace, self.asins)

        break
      except SellingApiForbiddenException as e:
        raise e
      except exceptions_to_retry as e:
        time.sleep(3)
      except SellingApiInvalidAsinException as e:
        matched = re.match(r'([A-Z0-9]{10}) is an invalid ASIN', e.message)
        if not matched:
          break

        asin = matched.groups()[0]
        if asin in self.asins:
          self.asins.remove(asin)

          if not self.asins:
            break
      except SellingApiBadRequestException:
        break
      except exceptions_not_retry as e:
        break

    if not products_info:
      if self.error_service:
        self.error_service.save_no_product_info_asins(
          "product_info", self.asins, self.marketplace
        )
      return

    for _, product_info in products_info.items():
      product_info['_id'] = product_info['asin']
      product_info['time'] = cur_time
    try:
      self.product_service.save_products(indice_name, list(products_info.values()))
    except Exception as e:
      logger.warning('[ProductSaveToServiceError] %s', products_info)
      logger.exception(e)
      raise e

  def search_catalog_items(self, spapi, marketplace, asins):
    response = spapi.search_catalog_items(asins, marketplace=marketplace)
    if not response:
      return

    return SpapiCatalogItemsParser.parse(response)
