import os
import time
import datetime

from dropshipping.utils.utils import is_asin_valid
from dropshipping.utils.offer_converters import EsLowestOfferListingOfferConverter
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.product_loaders.base import BaseProductLoader
from em_tasks import logger


class AsinLoader():
  pass


class OfferFileAsinLoader(AsinLoader):
  def __init__(self, asins_path, marketplace, offer_service, condition='new', ttl=5, force=False):
    if not asins_path:
      raise ValueError('asins_path can not be empty')

    asins_path = os.path.abspath(os.path.expanduser(asins_path))
    if not os.path.isfile(asins_path):
      raise ValueError('Could not find file {}'.format(asins_path))

    self.asins_path = asins_path
    self.marketplace = marketplace.lower()
    self.offer_service = offer_service
    self.offer_converter = EsLowestOfferListingOfferConverter()
    self.force = force

    if condition != 'new':
      condition = 'any'
    self.condition = condition

    self.ttl = ttl
    now = datetime.datetime.now()
    self.expire_date = now - datetime.timedelta(days=self.ttl)

  def load(self):
    offer_type = 'lowest_offer_listings'

    asins_buf = []
    batch_size = 500
    with open(self.asins_path, encoding='utf-8', errors='ignore') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        if not is_asin_valid(s):
          continue

        asins_buf.append(s)
        if len(asins_buf) < batch_size:
          continue

        asins = self.process_asins(asins_buf)
        for asin in asins:
          yield asin

        asins_buf = []

    if asins_buf:
      asins = self.process_asins(asins_buf)
      for asin in asins:
        yield asin

  def process_asins(self, asins):
    if self.force:
      return asins

    offers = None
    resp = self.offer_service.search_offers('lowest_offer_listings', asins, self.marketplace, self.condition)
    if resp:
      offers = self.offer_converter.convert(resp)
    if not offers and not isinstance(offers, dict):
      return asins

    asins_to_fetch_offer = {asin:None for asin in asins}
    for asin in asins:
      if asin not in offers or not offers[asin] or not offers[asin].get('time', None):
        continue

      offer_time = offers[asin]['time']
      if self.is_offer_expired(offer_time):
        continue

      asins_to_fetch_offer.pop(asin, None)

    if not asins_to_fetch_offer:
      return []

    return list(asins_to_fetch_offer.keys())

  def is_offer_expired(self, offer_time):
    if self.expire_date is None:
      return False

    if isinstance(offer_time, str):
      offer_time = datetime.datetime.strptime(offer_time, '%Y-%m-%dT%H:%M:%S')

    return offer_time < self.expire_date
